/*
 * Author:  Konrad Rzeszutek <conradus@iname.com>
 * Date:    Wed, Sept 22, 1999
 *
 * Note:
 *
 *  This object is responsible for manipulating all the drawing objects.
 *  It can manipulate all the drawing objects, add points, delete the
 *  the objects and clear the whole canvas.
 *
 */
package Editor2D;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;


public class MyCanvas extends Canvas {
    

    // Constants used for finding out what we are suppose to do. like we know?

    public static final int DRAW_LINE = 1;
    public static final int DRAW_CIRCLE = 2;
    public static final int DRAW_POLYGONS = 3;
    public static final int DRAW_CLEAR = 4;
    public static final int DRAW_FILL = 5;
    public static final int DRAW_DELETE = 6;
    public static final int DRAW_DUMP = 7;
    
    
    Dimension OffDim;          // get the diemensions of the screen
    Image OffImg;              // the image on which we will operate.
    Graphics OffG;             // offscreen drawing stuff
    Editor vi;                 // link to the main class.
    int sub_state;             // [PATTERN,THICK] -- how will the object be drawn.
    MyList vec;                // List of all the graphical primitives
    int draw_state;            // current selection


    /*
     * Initialize the constructor
     */
    public MyCanvas (Editor vi) {
	
	// constructor
	vec = new MyList();
	OffDim = null;
	OffImg = null;
	OffG = null;
	this.vi = vi;
	draw_state = DRAW_LINE;    // First thing that the user can draw.
    }
  

    /*
     * Clear the area
     *
     * require:
     *  x1, x2, y1, y2 != null
     *
     */
    public void ClearArea(int x1, int y1, int x2, int y2) {
	
	// clear area bounded by rectangle (x1, y1) -- (x2, y2)
    
	OffG.setColor(getBackground());    
	OffG.fillRect(0, 0, OffDim.width-1, OffDim.height-1);
	OffG.setColor(getForeground());
	
    }

    /*
     * Perform specific actions when a user selects a drawing object.
     *
     * require:
     *   ( (state==DRAW_LINE) || (state==DRAW_CIRCLE) || (DRAW_POLYGONS)
     *     (state==DRAW_DELETE) || (state==DRAW_CLEAR) || (DRAW_FILL) 
     *     (state==DRAW_DUMP))
     */
    public void perform(int state) {
	
	int i;  // a simple counter

	this.draw_state = state;

	switch (state) {
	    
	case DRAW_LINE: { // Add a new DrawLine.
	    vec.addItem(new DrawLine(OffG, OffDim,this,sub_state));
	    break;
	}	
	case DRAW_CIRCLE: {
	    vec.addItem(new DrawCircle(OffG,this,sub_state));
	    break;
	}
	case DRAW_POLYGONS: { 
	    vec.addItem(new DrawPolygons(OffG,OffDim,this,sub_state));
	    break;
	}
	case DRAW_FILL: { 
	     vec.addItem(new DrawFill(OffG,this,vec,sub_state));	    
	    break;
	}
	case DRAW_DUMP: { // Query each drawing object for its date.
	    for (i=0;i<= vec.length();i++)
		vi.outputData(vec.itemAt(i).dump() );
	    break;
	}
	case DRAW_CLEAR: {
	    vec.delAllElements();
	    OffG = null;
	    repaint();
	    draw_state = DRAW_LINE;
	    sub_state = MyPrimitives.DRAW_NO_PATTERN;
	    vi.reset();
	    break;
	}
	case DRAW_DELETE: {
	    // The user just selected this button, now he has to click 
	    // somewhere to specify what he wants to delete.
	}
	
	}
    }
    

    /*
     * Modify the current (and subsequent) drawing objects
     * of change of pattern/thickness.
     *
     * require:
     *   
     *    (state == MyPrimitives.DRAW_NO_PATTERN) || (state == MyPrimitives.DRAW_PATTERN_ON)
     *   || (state == MyPrimitives.DRAW_PATTERN_OFF) || (state == MyPrimitives.DRAW_THICK_ON)
     *   || (state == MyPrimitives.DRAW_THICK_OFF))
     */
    public void performSub(int state) {
	
	this.sub_state = state;
    
	if (vec.length()>= 0) { // If we have an item, then notify it.
	    vec.lastItem().setChanges(state);
	    sub_state = vec.lastItem().getState();
	}
    }

    /*
     * Redraws/draws the neccesary objects based on incoming
     * point and the state.
     *
     * require:
     *  pt instanceof Point
     */
    public void redraw(Point pt) {

	OffDim = getSize(); 
	int i; // Simple counter.
	// These three variables below are used for deleting only:
	int myDistance = -1;  // the shortest distance
	int currentD   =  0 ; 
	int delIndex = -1;   // index of the item to "remove"

	if (vec.length()!=-1) {
	    
	    switch (draw_state) {

	  case DRAW_LINE: {
	      if (vec.lastItem().canAdd())
		  vec.lastItem().add(pt);
	      else {
		  // Can't add, create a new class
		  vec.addItem(new DrawLine(OffG,OffDim,this,sub_state));
		  // Since we just added the item, it will be the last one.
		  vec.lastItem().add(pt);
	      }
	      break;
	  }
	  case DRAW_CIRCLE: {
	      if (vec.lastItem().canAdd())
		  vec.lastItem().add(pt);
	      else {
		  // Can't add, create a new class
		  vec.addItem(new DrawCircle(OffG,this,sub_state));
		  // Since we just added the item, it is the last one.
		  vec.lastItem().add(pt);
	      } 	      
	      break;
	  }
	  case DRAW_POLYGONS:{ 
	      vec.lastItem().add(pt);
	      break;
	  }
	    case DRAW_FILL: {
		vec.lastItem().add(pt); // Add this point to the current filling
		vec.addItem(new DrawFill(OffG,this,vec,sub_state)); // Create another filler for further objects.
		break;
	    }
	  case DRAW_DELETE: {
	      // Delete one of the items.
	      ClearArea(0, 0, OffDim.width, OffDim.height);
	      if (vec.length() >= 0) // Do we have anything to delete?
		  
		  myDistance = currentD = vec.itemAt(0).distance(pt);
	  
	      for (i=0;i<= vec.length();i++) {
		currentD = vec.itemAt(i).distance(pt);
		if ((currentD <= 500 && currentD <= myDistance)) {
		    myDistance =  currentD;
		    delIndex = i;
		  
		}
	      }
	      //System.out.println(myDistance);
	      if (delIndex >= 0) {
		  vec.itemAt(delIndex).delete(); // no removing from the queue, just marking them.
	      }
	      for (i=0;i<= vec.length();i++)		
		  vec.itemAt(i).draw();
	      break;
	  }
	  }
	  repaint();
      }

    }
  
  
  public void paint(Graphics g) {
    
      //    Primitive p;
    
    // make sure offscreen drawing stuff is initialized.  This stuff has to be
    // initialized here because the canvas' size is not yet valid in the
    // constructor
    if (OffG == null) {
      // get size of drawing area
      OffDim = getSize();   
      // create a place to draw offscreen
      OffImg = createImage(OffDim.width, OffDim.height); 
      // get graphics context
      OffG = OffImg.getGraphics();       
      
      // clear offscreen drawing area 
      ClearArea(0, 0, OffDim.width, OffDim.height);
      
    }    
    // draw onscreen from offscreen drawing area
    
    g.drawImage(OffImg, 0, 0, this);
    
  }


} // End of MyCanvas.java
