/*
 * Author: Konrad Rzeszutek <conradus@iname.com>
 * Date:   Tue, Sep 21 1999
 * File:   Editor2D/MyPrimitives.java
 *
 * Notes:
 *
 *  This is a fundamental class used for all drawing objects.
 *  It has some fundamental methods that are used throughout for all
 *  graphical objects.
 */
 
package Editor2D;
import java.awt.*;
import java.io.*;

public class MyPrimitives {

    // Constants for drawing 
    public static final int DRAW_NO_PATTERN = 0;
    public static final int DRAW_PATTERN_ON = 1;
    public static final int DRAW_PATTERN_OFF = 5;
    public static final int DRAW_THICK_ON = 2;
    public static final int DRAW_THICK_OFF = 6;

    // Huge numbers:
    public static final int POS_INFINITY =  99999999;
    public static final int NEG_INFINITY = -99999999;
    
    /*
      * Initializes the object.
      */
    public MyPrimitives() {
	
	myState = DRAW_NO_PATTERN;
	
	
    }
    /*
     * Adds a single point to the array of points.
     *
     * require:
     *   p instanceof Point
     *
     * note:
     *   if (!canAdd) then add() will invoke this.draw().
     */
    public void add(Point p) {

	if (nPoints != maxPoints) { // Do we have enough points?
	    xPoints[nPoints]=p.x;
	    yPoints[nPoints]=p.y;
	    nPoints++;
	    if (nPoints == maxPoints) this.draw(); // If so, let's draw.
	}
	
    }

    /*
     * Finds the distance between points in this object
     * and the given point.
     *
     * require:
     *  pt instanceof Point
     *
     * return
     *  0
     *
     * note:
     *  this method is not implemented.
     */
    public int distance(Point pt) {
	
    	return 0;
    }
    
    /* 
     * Draws the object.
     *
     * note:
     * this method is not implemented.
     */
    public void draw() {

    }

    /*
     * Changes the way the object is being drawn.
     *
     * require:
     *  (state == MyPrimitives.DRAW_NO_PATTERN) || (state == MyPrimitives.DRAW_PATTERN_ON)
     *   || (state == MyPrimitives.DRAW_PATTERN_OFF) || (state == MyPrimitives.DRAW_THICK_ON)
     *   || (state == MyPrimitives.DRAW_THICK_OFF))
     *
     */
    public void setChanges(int state) {
	   
   	if (state==DRAW_PATTERN_OFF) 
	    myState &= DRAW_THICK_ON;
	if (state==DRAW_THICK_OFF) 
	    myState &= DRAW_PATTERN_ON; 
	if ((state == DRAW_THICK_ON) || (state==DRAW_PATTERN_ON)) 	    
	    myState |= state; 
	
    }

    /*
     * Returns the way the object is being drawn.
     *
     * return:
     *  (getState() == MyPrimitives.DRAW_NO_PATTERN) || (getState() == MyPrimitives.DRAW_PATTERN_ON)
     *   || (getState() == MyPrimitives.DRAW_THICK_ON))
     *
     */
    public int getState() {
	
	return myState;
	
    }
    /*
     * Finds out if more points can be added.
     * 
     * return
     *  false or true.
     *
     */
    public boolean canAdd () {
	
	return (nPoints < maxPoints);
    }

    /*
     * Gives a string representation of the object's points.
     *
     * return:
     *  toString() instanceof String.
     *
     */
    public String toString () {

	String temp = "";    
	int i;
	if (deleted) temp+="[DELETED]";
	for (i=0; i<nPoints;i++) 
	    temp += "\n     ("+xPoints[i]+", "+yPoints[i]+")";
	return temp;
    }

    /*
     * Returns a representation of the objects actions and its points
     *
     * return:
     *  dump() instanceof String.
     */
    public String dump () {
    
	return this.toString();
    }

    /*
     * Gets the maximum current amount of edges(points)
     * that are contained within this object.
     *
     * return:
     *   0...n
     */
    public int getEdgeLength() {

	return nPoints;
    }
    /*
     * Returns a specific Point object for the requested edge.
     * 
     * require:
     *  0..n
     *
     * return:
     *  null or instancof Point
     * 
     */
    public Point getEdge(int index) {

	Point p = null;
	if (index <= nPoints-1) 
	    p = new Point (xPoints[index],yPoints[index]);
	return p;
    }
    
    /*
     * Is this a circle?
     * return:
     *  false
     */
    public boolean isCircle () {
	return false;
    }
    
    /*
     * Is this a polygon?
     * return:
     *  false
     */
    public boolean isPolygon() {
	return false;
    }
    
     /*
     * Mark this object as "deleted."
     *
     * note:
     *  draw() wont draw.
     */
    public void delete() {
	
	deleted = true;
    }
    
    /*
     * Simply query to see if the object is marked deleted.
     *
     * return:
     *  true || false
     */
    public boolean isDeleted() {
	
	return deleted;
    }
  
    // Variables that can be used in subclasses.
    
    int maxPoints = -1; // Max points we can accomodate.
    int nPoints = 0;    // points accumulated.
    int[] xPoints;      // The X's point "holder" -- holds only X points/
    int[] yPoints;      // The Y's point "holder" -- holds only Y points.
    int myState = 0;    // My current drawing behavior.
    boolean deleted = false;
    
}
